% This template is meant to work with Matlab and Octave. However, if your ICA algorithm resorts to Manopt, then Matlab may be required.

% ICA_for_photos
% Author: Michel Journe, 2007
% Modified by PA Absil, April 2007, April 2017

clear all
close all
%clc

%Sources
s_1=double(imread('5.tiff'));
s_2=double(imread('11.tiff'));

%s(1,:)=vect_matrice(s_1,1);
%s(2,:)=vect_matrice(s_2,1);

s(1,:) = transpose(s_1(:));  % turns s_1 into a long row vector
s(2,:) = transpose(s_2(:));  % turns s_1 into a long row vector

% Mixing matrix
%H=[0.4 -0.6;0.6 0.5];
H = [1.7, 1; .9, .6];
%H = orth(randn(2,2)) * diag([1,2]) * orth(randn(2,2));  % !!!

% Measured signals
x=H*s;

% Centering and scaling
x_std=(x-repmat((mean(x'))',1,length(x)))./repmat((std(x'))',1,length(x));

% PCA (Prewhitening)
R_x=cov(x_std');
[V,D]=eig(R_x);
H_PCA=V*sqrt(D);
z_PCA=inv(H_PCA)*x_std;

% ICA
% **************************************************************
% Replace the lines below by your ICA algorithm on the orthogonal group applied to the z_PCA data matrix.
%

% ** I believe that this is a JADE method (Cardoso)
%[z_ICA,Q]=tfuni4(z_PCA);
%H_ICA=H_PCA*Q';
%z_ICA=inv(H_ICA)*x_std;

% ** This is approximate joint diagonalization of SOBI matrices on the orthogonal group by steepest descent:
[z_ICA,V] = ICA_SOBI_ortho(z_PCA);

% ** This is just whitening:
%z_ICA = z_PCA;

% **************************************************************


% ** Plot figures
z_ICA=[1 0; 0 1]*z_ICA;  % Image independence is invariant if we take the negatives. We may need to put "-" signs here to avoid recovering negatives.

%z_ICA1=z_ICA(1,:)*255/(max(z_ICA(1,:))-min(z_ICA(1,:)))-min(z_ICA(1,:)*255/(max(z_ICA(1,:))-min(z_ICA(1,:))));
%z_ICA2=z_ICA(2,:)*255/(max(z_ICA(2,:))-min(z_ICA(2,:)))-min(z_ICA(2,:)*255/(max(z_ICA(2,:))-min(z_ICA(2,:))));
z_ICA1 = z_ICA(1,:);
z_ICA2 = z_ICA(2,:);

%z_1=vect_matrice(z_ICA1,size(s_1,2));
%z_2=vect_matrice(z_ICA2,size(s_2,2));
z_1 = reshape(z_ICA1,size(s_1));
z_2 = reshape(z_ICA2,size(s_2));

%x_1=vect_matrice(x(1,:),size(s_1,2));
%x_2=vect_matrice(x(2,:),size(s_2,2));
x_1 = reshape(x(1,:),size(s_1));
x_2 = reshape(x(2,:),size(s_2));


figure(1)
subplot(211)
colormap('gray');
%image(-x_1,'cdatamapping','scaled')
image(x_1,'cdatamapping','scaled')
ax=gca;
set(ax,'DataAspectRatio',[1,1,1],'DataAspectRatioMode','manual')
axis off
subplot(212) %figure(2)
colormap('gray');
image(x_2,'cdatamapping','scaled')
ax=gca;
set(ax,'DataAspectRatio',[1,1,1],'DataAspectRatioMode','manual')
axis off

figure(2)
subplot(211)
colormap('gray');
%image(-z_1,'cdatamapping','scaled')
image(z_1,'cdatamapping','scaled')
ax=gca;
set(ax,'DataAspectRatio',[1,1,1],'DataAspectRatioMode','manual')
axis off
subplot(212) %figure(4)
colormap('gray');
image(z_2,'cdatamapping','scaled')
ax=gca;
set(ax,'DataAspectRatio',[1,1,1],'DataAspectRatioMode','manual')
axis off
